# The LearningOnline Network
# Create a course
#
# $Id: loncreatecourse.pm,v 1.57 2004/05/11 06:02:58 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###

package Apache::loncreatecourse;

use strict;
use Apache::Constants qw(:common :http);
use Apache::lonnet;
use Apache::loncommon;
use Apache::lonratedt;
use Apache::londocs;
use Apache::lonlocal;
use Apache::londropadd;
use lib '/home/httpd/lib/perl';
use localenroll;

# ================================================ Get course directory listing

#FIXME - doesn't support directories under /userfiles/ BUG#2999
sub crsdirlist {
    my ($courseid,$which)=@_;
    unless ($which) { $which=''; }
    my %crsdata=&Apache::lonnet::coursedescription($courseid);
    my @listing=&Apache::lonnet::dirlist
	($which,$crsdata{'domain'},$crsdata{'num'},
	 &Apache::loncommon::propath($crsdata{'domain'},$crsdata{'num'}));
    my @output=();
    foreach (@listing) {
	#FIXME if list is a DIR need to recurse
	unless ($_=~/^\./) {
	    push (@output,(split(/\&/,$_))[0]);
	}
    }
    return @output;
}

# ============================================================= Read a userfile

sub readfile {
    my ($courseid,$which)=@_;
    my %crsdata=&Apache::lonnet::coursedescription($courseid);
    return &Apache::lonnet::getfile('/uploaded/'.$crsdata{'domain'}.'/'.
				    $crsdata{'num'}.'/'.$which);
}

# ============================================================ Write a userfile

sub writefile {
    (my $courseid, my $which,$ENV{'form.output'})=@_;
    my %crsdata=&Apache::lonnet::coursedescription($courseid);
    return &Apache::lonnet::finishuserfileupload(
					  $crsdata{'num'},$crsdata{'domain'},
					  $crsdata{'home'},
					  'output',$which);
}

# ===================================================================== Rewrite

sub rewritefile {
    my ($contents,%rewritehash)=@_;
    foreach (keys %rewritehash) {
	my $pattern=$_;
	$pattern=~s/(\W)/\\$1/gs;
	my $new=$rewritehash{$_};
	$contents=~s/$pattern/$new/gs;
    }
    return $contents;
}

# ============================================================= Copy a userfile

sub copyfile {
    my ($origcrsid,$newcrsid,$which)=@_;
    unless ($which=~/\.sequence$/) {
	return &writefile($newcrsid,$which,
		      &readfile($origcrsid,$which));
    } else {
	my %origcrsdata=&Apache::lonnet::coursedescription($origcrsid);
	my %newcrsdata= &Apache::lonnet::coursedescription($newcrsid);
	return &writefile($newcrsid,$which,
		 &rewritefile(
                     &readfile($origcrsid,$which),
	    (
       '/uploaded/'.$origcrsdata{'domain'}.'/'.$origcrsdata{'num'}.'/'
    => '/uploaded/'. $newcrsdata{'domain'}.'/'. $newcrsdata{'num'}.'/'
            )));
    }
}

# =============================================================== Copy a dbfile

sub copydb {
    my ($origcrsid,$newcrsid,$which)=@_;
    $which=~s/\.db$//;
    my %origcrsdata=&Apache::lonnet::coursedescription($origcrsid);
    my %newcrsdata= &Apache::lonnet::coursedescription($newcrsid);
    my %data=&Apache::lonnet::dump
	($which,$origcrsdata{'domain'},$origcrsdata{'num'});
    return &Apache::lonnet::put
	($which,\%data,$newcrsdata{'domain'},$newcrsdata{'num'});
}

# ========================================================== Copy resourcesdata

sub copyresourcedb {
    my ($origcrsid,$newcrsid)=@_;
    my %origcrsdata=&Apache::lonnet::coursedescription($origcrsid);
    my %newcrsdata= &Apache::lonnet::coursedescription($newcrsid);
    my %data=&Apache::lonnet::dump
	('resourcedata',$origcrsdata{'domain'},$origcrsdata{'num'});
    $origcrsid=~s/^\///;
    $origcrsid=~s/\//\_/;
    $newcrsid=~s/^\///;
    $newcrsid=~s/\//\_/;
    my %newdata=();
    undef %newdata;
    my $startdate=$data{$origcrsid.'.0.opendate'};
    my $today=time;
    my $delta=0;
    if ($startdate) {
	my $oneday=60*60*24;
	$delta=$today-$startdate;
	$delta=int($delta/$oneday)*$oneday;
    }
# ugly retro fix for broken version of types
    foreach (keys %data) {
	if ($_=~/\wtype$/) {
	    my $newkey=$_;
	    $newkey=~s/type$/\.type/;
	    $data{$newkey}=$data{$_};
	    delete $data{$_};
	}
    }
# adjust symbs
    my $pattern='uploaded/'.$origcrsdata{'domain'}.'/'.$origcrsdata{'num'}.'/';
    $pattern=~s/(\W)/\\$1/gs;
    my $new=    'uploaded/'. $newcrsdata{'domain'}.'/'. $newcrsdata{'num'}.'/';
    foreach (keys %data) {
	if ($_=~/$pattern/) {
	    my $newkey=$_;
	    $newkey=~s/$pattern/$new/;
	    $data{$newkey}=$data{$_};
	    delete $data{$_};
	}
    }
# adjust dates
    foreach (keys %data) {
	my $thiskey=$_;
	$thiskey=~s/^$origcrsid/$newcrsid/;
	$newdata{$thiskey}=$data{$_};
	if ($data{$_.'.type'}=~/^date/) {
	    $newdata{$thiskey}=$newdata{$thiskey}+$delta;
	}
    }
    return &Apache::lonnet::put
	('resourcedata',\%newdata,$newcrsdata{'domain'},$newcrsdata{'num'});
}

# ========================================================== Copy all userfiles

sub copyuserfiles {
    my ($origcrsid,$newcrsid)=@_;
    foreach (&crsdirlist($origcrsid,'userfiles')) {
	&copyfile($origcrsid,$newcrsid,$_);
    }
}
# ========================================================== Copy all userfiles

sub copydbfiles {
    my ($origcrsid,$newcrsid)=@_;
    foreach (&crsdirlist($origcrsid)) {
	if ($_=~/\.db$/) {
	    unless 
             ($_=~/^(nohist\_|discussiontimes|classlist|versionupdate|resourcedata)/) {
		 &copydb($origcrsid,$newcrsid,$_);
	     }
	}
    }
}

# ======================================================= Copy all course files

sub copycoursefiles {
    my ($origcrsid,$newcrsid)=@_;
    &copyuserfiles($origcrsid,$newcrsid);
    &copydbfiles($origcrsid,$newcrsid);
    &copyresourcedb($origcrsid,$newcrsid);
}

# ===================================================== Phase one: fill-in form

sub print_course_creation_page {
    my $r=shift;
    my $defdom=$ENV{'request.role.domain'};
    my %host_servers = &Apache::loncommon::get_library_servers($defdom);
    my $course_home = '<select name="course_home" size="1">'."\n";
    foreach my $server (sort(keys(%host_servers))) {
        $course_home .= qq{<option value="$server"};
        if ($server eq $Apache::lonnet::perlvar{'lonHostID'}) {
            $course_home .= " selected ";
        }
        $course_home .= qq{>$server $host_servers{$server}</option>};
    }
    $course_home .= "\n</select>\n";
    my $domform = &Apache::loncommon::select_dom_form($defdom,'ccdomain');
    my $bodytag=&Apache::loncommon::bodytag('Create a New Course');
    my $helplink=&Apache::loncommon::help_open_topic('Create_Course',&mt('Help on Creating Courses'));
    my $cloneform=&Apache::loncommon::select_dom_form
	($ENV{'request.role.domain'},'clonedomain').
		     &Apache::loncommon::selectcourse_link
		     ('ccrs','clonecourse','clonedomain');
    my $coursebrowserjs=&Apache::loncommon::coursebrowser_javascript();
    my $starttime = time;
    my $endtime = time+(6*30*24*60*60); # 6 months from now, approx
    my $date_table = &Apache::londropadd::date_setting_table($starttime,$endtime,'createcourse');
    my ($krbdef,$krbdefdom) =
    &Apache::loncommon::get_kerberos_defaults($defdom);
    my $javascript_validations=&Apache::londropadd::javascript_validations('createcourse',$krbdefdom);
    my %param = ( formname => 'document.ccrs',
                   kerb_def_dom => $krbdefdom,
                   kerb_def_auth => $krbdef
                );
    my $krbform = &Apache::loncommon::authform_kerberos(%param);
    my $intform = &Apache::loncommon::authform_internal(%param);
    my $locform = &Apache::loncommon::authform_local(%param);
    my %lt=&Apache::lonlocal::texthash(
		    'cinf' => "Course Information",
                    'ctit' => "Course Title",
                    'chsr' => "Course Home Server",
                    'cidn' => "Course ID/Number",
                    'opt'  => "optional",
                    'iinf' => "Institutional Information",
                    'stat' => "The following entries will be used to identify the course according to the naming scheme adopted by your institution. Your choices will be used to map an internal LON-CAPA course ID to the corresponding course section ID(s) used by the office responsible for providing official class lists for courses at your institution. This mapping is required if you choose to employ automatic population of class lists.",
                    'ccod' => "Course Code",
                    'toin' => "to interface with institutional data, e.g., fs03glg231 for Fall 2003 Geology 231",
                    'snid' => "Section Numbers and corresponding LON-CAPA section/group IDs",
                    'csli' => "a comma separated list of institutional section numbers, each separated by a colon from the (optional) corresponding section/group ID to be used in LON-CAPA e.g., 001:1,002:2",
                    'crcs' => "Crosslisted courses",
                    'cscs' => "a comma separated list of course sections crosslisted with the current course, with each entry including the institutional course section name followed by a colon and then the (optional) groupID to be used in LON-CAPA, e.g., fs03ent231001:ent1,fs03bot231001:bot1,fs03zol231002:bot2", 
                    'crco' => "Course Content",
                    'cncr' => "Completely new course",
                    'cecr' => "Clone an existing course", 
                    'map'  => "Map",
                    'smap' => "Select Map",
                    'sacr' => "Do NOT generate as standard course",
                    'ocik' => "only check if you know what you are doing",
                    'fres' => "First Resource",
                    'stco' => "standard courses only",
                    'blnk' => "Blank",
                    'sllb' => "Syllabus",
                    'navi' => "Navigate",
                    'cid'  => "Course ID",
                    'dmn'  => "Domain",
                    'asov' => "Additional settings, if specified below, will override cloned settings",
                    'assp' => "Assessment Parameters",
                    'oaas' => "Open all assessments",
                    'mssg' => "Messaging",
                    'scpf' => "Set course policy feedback to Course Coordinator",
                    'scfc' => "Set content feedback to Course Coordinator",
                    'cmmn' => "Communication",
                    'dsrd' => "Disable student resource discussion",
                    'dsuc' => "Disable student use of chatrooms",
                    'acco' => "Access Control",
                    'snak' => "Students need access key to enter course",
		    'kaut' => 
		    'Key authority (<tt>id@domain</tt>) if other than course',
                    'cc'   => "Course Coordinator",
                    'user' => "Username",
                    'ierc' => "Immediately expire own role as Course Coordinator",
                    'aens' => "Automated enrollment settings",
                    'aesc' => "The following settings control automatic enrollment of students in this class based on information available for this specific course from your institution's official classlists.",
                    'aadd' => "Automated adds",
                    'yes'  => "Yes",
                    'no'   => "No",
                    'audr' => "Automated drops",
                    'dacu' => "Duration of automated classlist updates",
                    'psam' => "Please select the authentication mechanism",
                    'pcda' => "Please choose the default authentication method to be used by new users added to this LON-CAPA domain by the automated enrollment process",
                    'nech' => "Notification of enrollment changes",
                    'nccl' => "Notification to course coordinator via LON-CAPA message when enrollment changes occur during the automated update?",
                    'irsp' => "Include retrieval of student photographs?",
		    'rshm' => 'Resource Space Home',
                    'opco' => "Open Course"
				       );
    $r->print(<<ENDDOCUMENT);
<html>
<script language="JavaScript" type="text/javascript">
var editbrowser = null;
function openbrowser(formname,elementname) {
    var url = '/res/?';
    if (editbrowser == null) {
        url += 'launch=1&';
    }
    url += 'catalogmode=interactive&';
    url += 'mode=edit&';
    url += 'form=' + formname + '&';
    url += 'element=' + elementname + '&';
    url += 'only=sequence' + '';
    var title = 'Browser';
    var options = 'scrollbars=1,resizable=1,menubar=0';
    options += ',width=700,height=600';
    editbrowser = open(url,title,options,'1');
    editbrowser.focus();
}
$javascript_validations
</script>
$coursebrowserjs
<head>
<title>The LearningOnline Network with CAPA</title>
</head>
$bodytag
$helplink
<form action="/adm/createcourse" method="post" name="ccrs">
<h2>$lt{'cinf'}</h2>
<p>
<b>$lt{'ctit'}:</b>
<input type="text" size="50" name="title">
</p><p>
<b>$lt{'chsr'}:</b>$course_home
</p><p>
<b>$lt{'cidn'} ($lt{'opt'})</b>
<input type="text" size="30" name="crsid">
</p><p>
<h2>$lt{'iinf'}</h2>
<p>
$lt{'stat'}
</p><p>
<b>$lt{'ccod'}</b>
<input type="text" size="30" name="crscode" /><br/>
($lt{'toin'})
</p><p>
<b>$lt{'snid'}</b>
 <input type="text" size="30" name="crssections" /><br/>
($lt{'csli'})
</p><p>
<b>$lt{'crcs'}</b>
<input type="text" size="30" name="crsxlist" /><br/>
($lt{'cscs'})
</p>
<h2>$lt{'crco'}</h2>
<table border="2">
<tr><th>$lt{'cncr'}</th><th>$lt{'cecr'}</th></tr>
<tr><td>
<p>
<b>$lt{'map'}:</b>
<input type="text" size="50" name="topmap">
<a href="javascript:openbrowser('ccrs','topmap')">$lt{'smap'}</a>
</p><p>
<b>$lt{'sacr'}</b><br /> 
($lt{'ocik'}):
<input type="checkbox" name="nonstandard">
</p>
<p>
<b>$lt{'fres'}</b><br />($lt{'stco'}):
<input type="radio" name="firstres" value="blank">$lt{'blnk'}
&nbsp;
<input type="radio" name="firstres" value="syl" checked>$lt{'sllb'}
&nbsp;
<input type="radio" name="firstres" value="nav">$lt{'navi'}
</p>
</td><td>
$lt{'cid'}: <input input type="text" size="25" name="clonecourse" value="" />
<br />
$lt{'dmn'}: 
$cloneform<br />&nbsp;<br />
$lt{'asov'}.
</td></tr>
</table>
<h2>$lt{'assp'}</h2>
<p>
<b>$lt{'oaas'}: </b>
<input type="checkbox" name="openall" />
</p>
<h2>$lt{'mssg'}</h2>
<p>
<b>$lt{'scpf'}: </b>
<input type="checkbox" name="setpolicy" checked>
<br />
<b>$lt{'scfc'}: </b>
<input type="checkbox" name="setcontent" checked>
</p>
<h2>$lt{'cmmn'}</h2>
<p>
<b>$lt{'dsrd'}: </b>
<input type="checkbox" name="disresdis" /> <br />
<b>$lt{'dsuc'}: </b>
<input type="checkbox" name="disablechat" />
</p>
<h2>$lt{'acco'}</h2>
<p>
<b>$lt{'snak'}: </b>
<input type="checkbox" name="setkeys" /><br />
<b>$lt{'kaut'}: </b>
<input type="text" size="30" name="keyauth" />
</p>
<h2>$lt{'rshm'}</h2>
<p>
<b>$lt{'rshm'}: </b>
<input type="text" name="reshome" size="30" value="/res/$defdom/" />
</p>
<p>
<h2>$lt{'aens'}</h2>
$lt{'aesc'}
</p>
<p>
<b>$lt{'aadd'}</b>
<input type="radio" name="autoadds" value="1" />$lt{'yes'} <input type="radio" name="autoadds" value="0" checked="true" />$lt{'no'}
</p><p>
<b>$lt{'audr'}</b>
<input type="radio" name="autodrops" value="1" />$lt{'yes'} <input type="radio" name="autodrops" value="0" checked="true" />$lt{'no'}
</p><p>
<b>$lt{'dacu'}</b>
$date_table
</p><p>
<b>$lt{'psam'}.</b><br />
$lt{'pcda'}.
</p><p>
$krbform
<br />
$intform
<br />
$locform
</p><p>
<b>$lt{'nech'}</b><br />
$lt{'nccl'}<br/>
<input type="radio" name="notify" value="1" />$lt{'yes'} <input type="radio" name="notify"
value="0" checked="true" />$lt{'no'}
</p><p>
<b>$lt{'irsp'}</b> <input type="radio" name="showphotos" value="1" />$lt{'yes'} <input type="radio" name="showphotos" value="0" checked="true" />$lt{'no'}
</p>
<hr />
<h2>$lt{'cc'}</h2>
<p>
<b>$lt{'user'}:</b> <input type="text" size="15" name="ccuname" />
</p><p>
<b>$lt{'dmn'}:</b> $domform
</p><p>
<b>$lt{'ierc'}:</b>
<input type="checkbox" name="expireown" checked>
</p>
<p>
<input type="hidden" name="phase" value="two" />
<input type="button" onClick="verify_message(this.form)" value="$lt{'opco'}">
</p>
</form>
</body>
</html>
ENDDOCUMENT
}

# ====================================================== Phase two: make course

sub create_course {
    my $r=shift;
    my $topurl='/res/'.&Apache::lonnet::declutter($ENV{'form.topmap'});
    my $ccuname=$ENV{'form.ccuname'};
    my $ccdomain=$ENV{'form.ccdomain'};
    $ccuname=~s/\W//g;
    $ccdomain=~s/\W//g;
    my $cdescr=$ENV{'form.title'};
    my $curl=$ENV{'form.topmap'};
    my $bodytag=&Apache::loncommon::bodytag('Create a New Course');
    $r->print(<<ENDENHEAD);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
</head>
$bodytag
ENDENHEAD
    #
    # Verify data
    #
    # Check the veracity of the course coordinator
    if (&Apache::lonnet::homeserver($ccuname,$ccdomain) eq 'no_host') {
	$r->print('<form action="/adm/createuser" method="post" name="crtuser">');
        $r->print(&mt('No such user').' '.$ccuname.' '.&mt('at').' '.$ccdomain.'.<br />');
	$r->print(&mt("Please click Back on your browser and select another user, or "));
	$r->print('
	    <input type="hidden" name="phase" value="get_user_info" />
            <input type="hidden" name="ccuname" value="'.$ccuname.'" />
            <input type="hidden" name="ccdomain" value="'.$ccdomain.'" />
            <input name="userrole" type="submit" value="'.
		  &mt('Create User').'" />
	</form></body></html>');
	return;
    }
    # Check the proposed home server for the course
    my %host_servers = &Apache::loncommon::get_library_servers
        ($ENV{'request.role.domain'});
    if (! exists($host_servers{$ENV{'form.course_home'}})) {
        $r->print(&mt('Invalid home server for course').': '.
                  $ENV{'form.course_home'}.'</body></html>');
        return;
    }
#
# Open course
#
    my %cenv=();
    my $courseid=&Apache::lonnet::createcourse($ENV{'request.role.domain'},
                                               $cdescr,$curl,
                                               $ENV{'form.course_home'},
                                               $ENV{'form.nonstandard'});

    # Note: The testing routines depend on this being output; see 
    # Utils::Course. This needs to at least be output as a comment
    # if anyone ever decides to not show this, and Utils::Course::new
    # will need to be suitably modified.
    $r->print('New LON-CAPA Course ID: '.$courseid.'<br>');
#
# Check if created correctly
#
    my ($crsudom,$crsunum)=($courseid=~/^\/(\w+)\/(\w+)$/);
    my $crsuhome=&Apache::lonnet::homeserver($crsunum,$crsudom);
    $r->print(&mt('Created on').': '.$crsuhome.'<br>');
#
# Are we cloning?
#
    my $cloneid='';
    if (($ENV{'form.clonecourse'}) && ($ENV{'form.clonedomain'})) {
	$cloneid='/'.$ENV{'form.clonedomain'}.'/'.$ENV{'form.clonecourse'};
        my ($clonecrsudom,$clonecrsunum)=($cloneid=~/^\/(\w+)\/(\w+)$/);
	my $clonehome=&Apache::lonnet::homeserver($clonecrsunum,$clonecrsudom);
	if ($clonehome eq 'no_host') {
	    $r->print(
    '<br /><font color="red">'.&mt('Attempting to clone non-existing course').' '.$cloneid.'</font>');
	} else {
	    $r->print(
    '<br /><font color="green">'.&mt('Cloning course from').' '.$clonehome.'</font>');
	    my %oldcenv=&Apache::lonnet::dump('environment',$crsudom,$crsunum);
# Copy all files
	    &copycoursefiles($cloneid,$courseid);
# Restore URL
	    $cenv{'url'}=$oldcenv{'url'};
# Restore title
	    $cenv{'description'}=$oldcenv{'description'};
# Mark as cloned
	    $cenv{'clonedfrom'}=$cloneid;
	    delete($cenv{'default_enrollment_start_date'});
	    delete($cenv{'default_enrollment_end_date'});
	}
    }
#
# Set environment (will override cloned, if existing)
#
    my @affiliates = ();  # Used to accumulate sections and crosslistings
    if ($ENV{'form.crsid'}) {
        $cenv{'courseid'}=$ENV{'form.crsid'};
    }
    if ($ENV{'form.crscode'}) {
        $cenv{'internal.coursecode'}=$ENV{'form.crscode'};
    }
    if ($ENV{'form.crssections'}) {
        $cenv{'internal.sectionnums'}=$ENV{'form.crssections'};
        my @sections = ();
        if ($cenv{'internal.sectionnums'} =~ m/,/) {
            @sections = split/,/,$cenv{'internal.sectionnums'};
        } else {
            $sections[0] = $cenv{'internal.sectionnums'};
        }
        if (@sections > 0) {
            foreach (@sections) {
                my ($sec,$gp) = split/:/,$_; 
                push @affiliates,$ENV{'form.crscode'}.$sec;
            }
        }
    }
# do not hide course coordinator from staff listing, 
# even if privileged
    $cenv{'nothideprivileged'}=$ccuname.':'.$ccdomain;
    if ($ENV{'form.crsxlist'}) {
       $cenv{'internal.crosslistings'}=$ENV{'form.crsxlist'};
       my @xlists = ();
        if ($cenv{'internal.crosslistings'} =~ m/,/) {
            @xlists = split/,/,$cenv{'internal.crosslistings'};
        } else {
            $xlists[0] = $cenv{'internal.crosslistings'};
        }
        if (@xlists > 0) {
            foreach (@xlists) {
                my ($xl,$gp) = split/:/,$_;
                push @affiliates,$xl;
            }
        }
    }
    if ($ENV{'form.autoadds'}) {
        $cenv{'internal.autoadds'}=$ENV{'form.autoadds'};
    }
    if ($ENV{'form.autodrops'}) {
        $cenv{'internal.autodrops'}=$ENV{'form.autodrops'};
    }
    if ($ENV{'form.notify'}) {
      if ($ccuname) {
        $cenv{'internal.notifylist'} = $ccuname;
      }
    }
    if ($ccuname) {
        $cenv{'internal.courseowner'} = $ccuname;
    } else {
        $cenv{'internal.courseowner'} = $ENV{'user.name'};
    }
    if (@affiliates > 0) {
        my @badclasses = ();
        foreach my $class (@affiliates) {
            my $addcheck =  &localenroll::new_course($class,$cenv{'internal.courseowner'});
            unless ($addcheck eq 'ok') {
                push @badclasses, $class; 
            }
        }
        if (@badclasses > 0) {
            $r->print('<font color="red">'.
                      "The courses listed below were included as sections or crosslistings affiliated with your new LON-CAPA course. If automated course roster updates are enabled for this class, these particular sections/crosslistings will not contribute towards enrollment, because the user identified as the course owner for this LON-CAPA course ($cenv{'internal.courseowner'}) - does not have rights to access enrollment in these classes (as determined by your instititution's policies on access to official classlists).<br/><ul>\n");
            foreach (@badclasses) {
                $r->print("<li>$_</li>\n");
            }
            $r->print ("</ul><br/><br/></font>\n");
        }
    }
    my $startdate = &Apache::lonhtmlcommon::get_date_from_form('startdate');
    my $enddate   = &Apache::lonhtmlcommon::get_date_from_form('enddate');
    if ($ENV{'form.no_end_date'}) {
      $enddate = 0;
    }
    $cenv{'internal.autostart'}=$startdate;
    $cenv{'internal.autoend'}=$enddate;
    if ($ENV{'form.showphotos'}) {
      $cenv{'internal.showphotos'}=$ENV{'form.showphotos'};
    }
    if ($ENV{'form.login'} eq 'krb') {
        $cenv{'internal.authtype'} = 'krb';
        $cenv{'internal.authtype'} .=$ENV{'form.krbver'};
        $cenv{'internal.autharg'} = $ENV{'form.krbarg'};
    } elsif ($ENV{'form.login'} eq 'int') {
        $cenv{'internal.authtype'} ='internal';
        if ((defined($ENV{'form.intarg'})) && ($ENV{'form.intarg'})) {
            $cenv{'internal.autharg'} = $ENV{'form.intarg'};
        }
    } elsif ($ENV{'form.login'} eq 'loc') {
        $cenv{'internal.authtype'} = 'localauth';
        if ((defined($ENV{'form.locarg'})) && ($ENV{'form.locarg'})) {
            $cenv{'internal.autharg'} = $ENV{'form.locarg'};
        }
    }
    if ( ($cenv{'internal.authtype'} =~ /^krb/) && ($cenv{'internal.autoadds'} == 1)) {
        if (! defined($cenv{'internal.autharg'}) || $cenv{'internal.autharg'}  eq '') {
            $r->print('<font color="red" size="+1">'.
                      &mt('As you did not include the default Kerberos domain to be used for authentication in this class, the institutional data used by the automated enrollment process must include the Kerberos domain for each new student').'</font></p>');
        }
    }
    if (($ccdomain) && ($ccuname)) {
       if ($ENV{'form.setpolicy'}) {
           $cenv{'policy.email'}=$ccuname.':'.$ccdomain;
       }
       if ($ENV{'form.setcontent'}) {
           $cenv{'question.email'}=$ccuname.':'.$ccdomain;
       }
    }
    if ($ENV{'form.reshome'}) {
	$cenv{'reshome'}=$ENV{'form.reshome'}.'/';
	$cenv{'reshome'}=~s/\/+$/\//;
    }
#
# course has keyed access
#
    if ($ENV{'form.setkeys'}) {
       $cenv{'keyaccess'}='yes';
    }
# if specified, key authority is not course, but user
# only active if keyaccess is yes
    if ($ENV{'form.keyauth'}) {
	$ENV{'form.keyauth'}=~s/[^\w\@]//g;
	if ($ENV{'form.keyauth'}) {
	    $cenv{'keyauth'}=$ENV{'form.keyauth'};
	}
    }

    if ($ENV{'form.disresdis'}) {
        $cenv{'pch.roles.denied'}='st';
    }
    if ($ENV{'form.disablechat'}) {
        $cenv{'plc.roles.denied'}='st';
    }

    # Record we've not yet viewed the Course Initialization Helper for this 
    # course
    $cenv{'course.helper.not.run'} = 1;
    #
    # Use new Randomseed
    #
    $cenv{'rndseed'}=&Apache::lonnet::latest_rnd_algorithm_id();;
    $cenv{'receiptalg'}=&Apache::lonnet::latest_receipt_algorithm_id();;
    #
    # The encryption code and receipt prefix for this course
    #
    $cenv{'internal.encseed'}=$Apache::lonnet::perlvar{'lonReceipt'}.$$.time.int(rand(9999));
    $cenv{'internal.encpref'}=100+int(9*rand(99));
    #
    # By default, use standard grading
    $cenv{'grading'} = 'standard';

    $r->print('<br />'.&mt('Setting environment').': '.                 
          &Apache::lonnet::put('environment',\%cenv,$crsudom,$crsunum).'<br>');
#
# Open all assignments
#
    if ($ENV{'form.openall'}) {
       my $storeunder=$crsudom.'_'.$crsunum.'.0.opendate';
       my %storecontent = ($storeunder         => time,
                           $storeunder.'.type' => 'date_start');
       
       $r->print(&mt('Opening all assignments').': '.&Apache::lonnet::cput
                 ('resourcedata',\%storecontent,$crsudom,$crsunum).'<br>');
   }
#
# Set first page
#
    unless (($ENV{'form.nonstandard'}) || ($ENV{'form.firstres'} eq 'blank')
	    || ($cloneid)) {
	$r->print(&mt('Setting first resource').': ');
        my ($errtext,$fatal)=
           &Apache::londocs::mapread($crsunum,$crsudom,'default.sequence');
        $r->print(($fatal?$errtext:'read ok').' - ');
        my $title; my $url;
        if ($ENV{'form.firstres'} eq 'syl') {
	    $title='Syllabus';
            $url='/public/'.$crsudom.'/'.$crsunum.'/syllabus';
        } else {
            $title='Navigate Contents';
            $url='/adm/navmaps';
        }
        $Apache::lonratedt::resources[1]=$title.':'.$url.':false:start:res';
        ($errtext,$fatal)=
           &Apache::londocs::storemap($crsunum,$crsudom,'default.sequence');
        $r->print(($fatal?$errtext:'write ok').'<br>');
  }
#
# Make current user course adminstrator
#
    my $end=undef;
    my $addition='';
    if ($ENV{'form.expireown'}) { $end=time+5; $addition='expired'; }
    $r->print(&mt('Assigning').' '.$addition.' '.&mt('role of course coordinator to self').': '.
    &Apache::lonnet::assignrole(
     $ENV{'user.domain'},$ENV{'user.name'},$courseid,'cc',$end).'<br>');
#
# Make additional user course administrator
#
   if (($ccdomain) && ($ccuname)) {
    $r->print(&mt('Assigning role of course coordinator to').' '.
               $ccuname.' at '.$ccdomain.': '.
    &Apache::lonnet::assignrole($ccdomain,$ccuname,$courseid,'cc').'<p>');
   }
    if ($ENV{'form.setkeys'}) {
	$r->print(
 '<p><a href="/adm/managekeys?cid='.$crsudom.'_'.$crsunum.'">'.&mt('Manage Access Keys').'</a></p>');
    }
# Flush the course logs so reverse user roles immediately updated
    &Apache::lonnet::flushcourselogs();
    $r->print('<p>'.&mt('Roles will be active at next login').'.</p></body></html>');
}

# ===================================================================== Handler
sub handler {
    my $r = shift;

    if ($r->header_only) {
       &Apache::loncommon::content_type($r,'text/html');
       $r->send_http_header;
       return OK;
    }

    if (&Apache::lonnet::allowed('ccc',$ENV{'request.role.domain'})) {
       &Apache::loncommon::content_type($r,'text/html');
       $r->send_http_header;

       if ($ENV{'form.phase'} eq 'two') {
           &create_course($r);
       } else {
	   &print_course_creation_page($r);
       }
   } else {
      $ENV{'user.error.msg'}=
        "/adm/createcourse:ccc:0:0:Cannot create courses";
      return HTTP_NOT_ACCEPTABLE; 
   }
   return OK;
} 

1;
__END__
