# The LearningOnline Network
# Preferences
#
# $Id: lonpreferences.pm,v 1.44 2004/06/08 01:32:02 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# This package uses the "londes.js" javascript code. 
#
# TODOs that have to be completed:
#    interface with lonnet to change the password
 
package Apache::lonpreferences;

use strict;
use Apache::Constants qw(:common);
use Apache::File;
use Crypt::DES;
use DynaLoader; # for Crypt::DES version
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::lonlocal;

#
# Write lonnet::passwd to do the call below.
# Use:
#   my $answer=reply("encrypt:passwd:$udom:$uname:$upass",$tryserver);
#
##################################################
#          password associated functions         #
##################################################
sub des_keys {
    # Make a new key for DES encryption.
    # Each key has two parts which are returned separately.
    # Please note:  Each key must be passed through the &hex function
    # before it is output to the web browser.  The hex versions cannot
    # be used to decrypt.
    my @hexstr=('0','1','2','3','4','5','6','7',
                '8','9','a','b','c','d','e','f');
    my $lkey='';
    for (0..7) {
        $lkey.=$hexstr[rand(15)];
    }
    my $ukey='';
    for (0..7) {
        $ukey.=$hexstr[rand(15)];
    }
    return ($lkey,$ukey);
}

sub des_decrypt {
    my ($key,$cyphertext) = @_;
    my $keybin=pack("H16",$key);
    my $cypher;
    if ($Crypt::DES::VERSION>=2.03) {
        $cypher=new Crypt::DES $keybin;
    } else {
        $cypher=new DES $keybin;
    }
    my $plaintext=
	$cypher->decrypt(unpack("a8",pack("H16",substr($cyphertext,0,16))));
    $plaintext.=
	$cypher->decrypt(unpack("a8",pack("H16",substr($cyphertext,16,16))));
    $plaintext=substr($plaintext,1,ord(substr($plaintext,0,1)) );
    return $plaintext;
}

################################################################
#                       Handler subroutines                    #
################################################################

################################################################
#         Language Change Subroutines                          #
################################################################

sub wysiwygchanger {
    my $r = shift;
    my %userenv = &Apache::lonnet::get
        ('environment',['wysiwygeditor']);
    my $offselect='';
    my $onselect='checked="1"';
    if ($userenv{'wysiwygeditor'}) {
	$onselect='';
	$offselect='checked="1"';
    }
    my $switchoff=&mt('Disable WYSIWYG editor');
    my $switchon=&mt('Enable WYSIWYG editor');
    $r->print(<<ENDLSCREEN);
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="set_wysiwyg" />
<br />
<input type="radio" name="wysiwyg" value="off" $onselect /> $switchoff<br />
<input type="radio" name="wysiwyg" value="on" $offselect /> $switchon
ENDLSCREEN
    $r->print('<br /><input type="submit" value="'.&mt('Change').'" />');
}


sub verify_and_change_wysiwyg {
    my $r = shift;
    my $newsetting=$ENV{'form.wysiwyg'};
    &Apache::lonnet::put('environment',{'wysiwygeditor' => $newsetting});
    &Apache::lonnet::appenv('environment.wysiwygeditor' => $newsetting);
    $r->print('<p>'.&mt('Setting WYSIWYG editor to:').' '.&mt($newsetting).'</p>');
}

################################################################
#         Language Change Subroutines                          #
################################################################
sub languagechanger {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my %userenv = &Apache::lonnet::get
        ('environment',['languages']);
    my $language=$userenv{'languages'};

    my $pref=&mt('Preferred language');
    my %langchoices=('' => 'No language preference');
    foreach (&Apache::loncommon::languageids()) {
	if (&Apache::loncommon::supportedlanguagecode($_)) {
	    $langchoices{&Apache::loncommon::supportedlanguagecode($_)}
	               = &Apache::loncommon::plainlanguagedescription($_);
	}
    }
    my $selectionbox=&Apache::loncommon::select_form($language,'language',
						     %langchoices);
    $r->print(<<ENDLSCREEN);
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="verify_and_change_languages" />
<br />$pref: $selectionbox
ENDLSCREEN
    $r->print('<br /><input type="submit" value="'.&mt('Change').'" />');
}


sub verify_and_change_languages {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
# Screenname
    my $newlanguage  = $ENV{'form.language'};
    $newlanguage=~s/[^\-\w]//g;
    my $message='';
    if ($newlanguage) {
        &Apache::lonnet::put('environment',{'languages' => $newlanguage});
        &Apache::lonnet::appenv('environment.languages' => $newlanguage);
        $message='Set new preferred languages to '.$newlanguage;
    } else {
        &Apache::lonnet::del('environment',['languages']);
        &Apache::lonnet::delenv('environment\.languages');
        $message='Reset preferred language';
    }
    $r->print(<<ENDVCSCREEN);
</p>
$message
ENDVCSCREEN
}


################################################################
#         Anonymous Discussion Name Change Subroutines         #
################################################################
sub screennamechanger {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my %userenv = &Apache::lonnet::get
        ('environment',['screenname','nickname']);
    my $screenname=$userenv{'screenname'};
    my $nickname=$userenv{'nickname'};
    $r->print(<<ENDSCREEN);
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="verify_and_change_screenname" />
<br />New screenname (shown if you post anonymously):
<input type="text" size="20" value="$screenname" name="screenname" />
<br />New nickname (shown if you post non-anonymously):
<input type="text" size="20" value="$nickname" name="nickname" />
<input type="submit" value="Change" />
</form>
ENDSCREEN
}

sub verify_and_change_screenname {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
# Screenname
    my $newscreen  = $ENV{'form.screenname'};
    $newscreen=~s/[^ \w]//g;
    my $message='';
    if ($newscreen) {
        &Apache::lonnet::put('environment',{'screenname' => $newscreen});
        &Apache::lonnet::appenv('environment.screenname' => $newscreen);
        $message='Set new screenname to '.$newscreen;
    } else {
        &Apache::lonnet::del('environment',['screenname']);
        &Apache::lonnet::delenv('environment\.screenname');
        $message='Reset screenname';
    }
# Nickname
    $message.='<br />';
    $newscreen  = $ENV{'form.nickname'};
    $newscreen=~s/[^ \w]//g;
    if ($newscreen) {
        &Apache::lonnet::put('environment',{'nickname' => $newscreen});
        &Apache::lonnet::appenv('environment.nickname' => $newscreen);
        $message.='Set new nickname to '.$newscreen;
    } else {
        &Apache::lonnet::del('environment',['nickname']);
        &Apache::lonnet::delenv('environment\.nickname');
        $message.='Reset nickname';
    }

    $r->print(<<ENDVCSCREEN);
</p>
$message
ENDVCSCREEN
}

################################################################
#         Message Forward                                      #
################################################################

sub msgforwardchanger {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my %userenv = &Apache::lonnet::get('environment',['msgforward','notification','critnotification']);
    my $msgforward=$userenv{'msgforward'};
    my $notification=$userenv{'notification'};
    my $critnotification=$userenv{'critnotification'};
    my $forwardingHelp = Apache::loncommon::help_open_topic("Prefs_Forwarding",
							    "What are forwarding ".
							    "and notification ".
							    "addresses");
    my $criticalMessageHelp = Apache::loncommon::help_open_topic("Course_Critical_Message",
								 "What are critical messages");

    $r->print(<<ENDMSG);
$forwardingHelp <br />
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="verify_and_change_msgforward" />
New Forwarding Address(es) (<tt>user:domain,user:domain,...</tt>):
<input type="text" size="40" value="$msgforward" name="msgforward" /><hr />
New Message Notification Email Address(es) (<tt>joe\@doe.com,jane\@doe.edu,...</tt>):
<input type="text" size="40" value="$notification" name="notification" /><hr />
New Critical Message Notification Email Address(es) (<tt>joe\@doe.com,jane\@doe.edu,...</tt>):
<input type="text" size="40" value="$critnotification" name="critnotification" />$criticalMessageHelp<hr />
<input type="submit" value="Change" />
</form>
ENDMSG
}

sub verify_and_change_msgforward {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my $newscreen  = '';
    my $message='';
    foreach (split(/\,/,$ENV{'form.msgforward'})) {
	my ($msuser,$msdomain)=split(/[\@\:]/,$_);
        $msuser=~s/\W//g;
        $msdomain=~s/\W//g;
        if (($msuser) && ($msdomain)) {
	    if (&Apache::lonnet::homeserver($msuser,$msdomain) ne 'no_host') {
               $newscreen.=$msuser.':'.$msdomain.',';
	   } else {
               $message.='No such user: '.$msuser.':'.$msdomain.'<br>';
           }
        }
    }
    $newscreen=~s/\,$//;
    if ($newscreen) {
        &Apache::lonnet::put('environment',{'msgforward' => $newscreen});
        &Apache::lonnet::appenv('environment.msgforward' => $newscreen);
        $message.='Set new message forwarding to '.$newscreen.'<br />';
    } else {
        &Apache::lonnet::del('environment',['msgforward']);
        &Apache::lonnet::delenv('environment\.msgforward');
        $message.='Reset message forwarding<br />';
    }
    my $notification=$ENV{'form.notification'};
    $notification=~s/\s//gs;
    if ($notification) {
        &Apache::lonnet::put('environment',{'notification' => $notification});
        &Apache::lonnet::appenv('environment.notification' => $notification);
        $message.='Set message notification address to '.$notification.'<br />';
    } else {
        &Apache::lonnet::del('environment',['notification']);
        &Apache::lonnet::delenv('environment\.notification');
        $message.='Reset message notification<br />';
    }
    my $critnotification=$ENV{'form.critnotification'};
    $critnotification=~s/\s//gs;
    if ($critnotification) {
        &Apache::lonnet::put('environment',{'critnotification' => $critnotification});
        &Apache::lonnet::appenv('environment.critnotification' => $critnotification);
        $message.='Set critical message notification address to '.$critnotification;
    } else {
        &Apache::lonnet::del('environment',['critnotification']);
        &Apache::lonnet::delenv('environment\.critnotification');
        $message.='Reset critical message notification<br />';
    }
    $r->print(<<ENDVCMSG);
</p>
$message
ENDVCMSG
}

################################################################
#         Colors                                               #
################################################################

sub colorschanger {
    my $r = shift;
# figure out colors
    my $function='student';
    if ($ENV{'request.role'}=~/^(cc|in|ta|ep)/) {
	$function='coordinator';
    }
    if ($ENV{'request.role'}=~/^(su|dc|ad|li)/) {
	$function='admin';
    }
    if (($ENV{'request.role'}=~/^(au|ca)/) ||
	($ENV{'REQUEST_URI'}=~/^(\/priv|\~)/)) {
	$function='author';
    }
    my $domain=&Apache::loncommon::determinedomain();
    my %colortypes=('pgbg'  => 'Page Background',
                    'tabbg' => 'Header Background',
                    'sidebg'=> 'Header Border',
                    'font'  => 'Font',
                    'link'  => 'Un-Visited Link',
                    'vlink' => 'Visited Link',
                    'alink' => 'Active Link');
    my $chtable='';
    foreach my $item (sort(keys(%colortypes))) {
       my $curcol=&Apache::loncommon::designparm($function.'.'.$item,$domain);
       $chtable.='<tr><td>'.$colortypes{$item}.'</td><td bgcolor="'.$curcol.
        '">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td><input name="'.$item.
        '" size="10" value="'.$curcol.
'" /></td><td><a href="javascript:pjump('."'color_custom','".$colortypes{$item}.
"','".$curcol."','"
	    .$item."','parmform.pres','psub'".');">Select</a></td></tr>';
    }
    my $pjump_def = &Apache::lonhtmlcommon::pjump_javascript_definition();
    $r->print(<<ENDCOL);
<script>

    function pclose() {
        parmwin=window.open("/adm/rat/empty.html","LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");
        parmwin.close();
    }

    $pjump_def

    function psub() {
        pclose();
        if (document.parmform.pres_marker.value!='') {
            if (document.parmform.pres_type.value!='') {
                eval('document.server.'+
                     document.parmform.pres_marker.value+
		     '.value=document.parmform.pres_value.value;');
	    }
        } else {
            document.parmform.pres_value.value='';
            document.parmform.pres_marker.value='';
        }
    }


</script>
<form name="parmform">
<input type="hidden" name="pres_marker" />
<input type="hidden" name="pres_type" />
<input type="hidden" name="pres_value" />
</form>
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="verify_and_change_colors" />
<table border="2">
$chtable
</table>
<input type="submit" value="Change Custom Colors" />
<input type="submit" name="resetall" value="Reset All Colors to Default" />
</form>
ENDCOL
}

sub verify_and_change_colors {
    my $r = shift;
# figure out colors
    my $function='student';
    if ($ENV{'request.role'}=~/^(cc|in|ta|ep)/) {
	$function='coordinator';
    }
    if ($ENV{'request.role'}=~/^(su|dc|ad|li)/) {
	$function='admin';
    }
    if (($ENV{'request.role'}=~/^(au|ca)/) ||
	($ENV{'REQUEST_URI'}=~/^(\/priv|\~)/)) {
	$function='author';
    }
    my $domain=&Apache::loncommon::determinedomain();
    my %colortypes=('pgbg'  => 'Page Background',
                    'tabbg' => 'Header Background',
                    'sidebg'=> 'Header Border',
                    'font'  => 'Font',
                    'link'  => 'Un-Visited Link',
                    'vlink' => 'Visited Link',
                    'alink' => 'Active Link');

    my $message='';
    foreach my $item (keys %colortypes) {
        my $color=$ENV{'form.'.$item};
        my $entry='color.'.$function.'.'.$item;
	if (($color=~/^\#[0-9A-Fa-f]{6}$/) && (!$ENV{'form.resetall'})) {
	    &Apache::lonnet::put('environment',{$entry => $color});
	    &Apache::lonnet::appenv('environment.'.$entry => $color);
	    $message.='Set '.$colortypes{$item}.' to '.$color.'<br />';
	} else {
	    &Apache::lonnet::del('environment',[$entry]);
	    &Apache::lonnet::delenv('environment\.'.$entry);
	    $message.='Reset '.$colortypes{$item}.'<br />';
	}
    }
    $r->print(<<ENDVCCOL);
</p>
$message
<form name="client" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="changecolors" />
</form>
ENDVCCOL
}

######################################################
#            password handler subroutines            #
######################################################
sub passwordchanger {
    # This function is a bit of a mess....
    # Passwords are encrypted using londes.js (DES encryption)
    my $r = shift;
    my $errormessage = shift;
    $errormessage = ($errormessage || '');
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my $homeserver = $ENV{'user.home'};
    my $currentauth=&Apache::lonnet::queryauthenticate($user,$domain);
    # Check for authentication types that allow changing of the password.
    return if ($currentauth !~ /^(unix|internal):/);
    #
    # Generate keys
    my ($lkey_cpass ,$ukey_cpass ) = &des_keys();
    my ($lkey_npass1,$ukey_npass1) = &des_keys();
    my ($lkey_npass2,$ukey_npass2) = &des_keys();
    # Store the keys in the log files
    my $lonhost = $r->dir_config('lonHostID');
    my $logtoken=Apache::lonnet::reply('tmpput:'
				       .$ukey_cpass  . $lkey_cpass .'&'
				       .$ukey_npass1 . $lkey_npass1.'&'
				       .$ukey_npass2 . $lkey_npass2,
				       $lonhost);
    # Hexify the keys for output as javascript variables
    $ukey_cpass = hex($ukey_cpass);
    $lkey_cpass = hex($lkey_cpass);
    $ukey_npass1= hex($ukey_npass1);
    $lkey_npass1= hex($lkey_npass1);
    $ukey_npass2= hex($ukey_npass2);
    $lkey_npass2= hex($lkey_npass2);
    # Output javascript to deal with passwords
    # Output DES javascript
    $r->print("<html><head>");
    {
	my $include = $r->dir_config('lonIncludes');
	my $jsh=Apache::File->new($include."/londes.js");
	$r->print(<$jsh>);
    }
    $r->print(<<ENDFORM);
<script language="JavaScript">

    function send() {
        uextkey=this.document.client.elements.ukey_cpass.value;
        lextkey=this.document.client.elements.lkey_cpass.value;
        initkeys();

        this.document.server.elements.currentpass.value
            =crypted(this.document.client.elements.currentpass.value);

        uextkey=this.document.client.elements.ukey_npass1.value;
        lextkey=this.document.client.elements.lkey_npass1.value;
        initkeys();
        this.document.server.elements.newpass_1.value
            =crypted(this.document.client.elements.newpass_1.value);

        uextkey=this.document.client.elements.ukey_npass2.value;
        lextkey=this.document.client.elements.lkey_npass2.value;
        initkeys();
        this.document.server.elements.newpass_2.value
            =crypted(this.document.client.elements.newpass_2.value);

        this.document.server.submit();
    }

</script>
$errormessage

<p>
<!-- We separate the forms into 'server' and 'client' in order to
     ensure that unencrypted passwords will not be sent out by a
     crappy browser -->

<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="logtoken"    value="$logtoken" />
<input type="hidden" name="action"      value="verify_and_change_pass" />
<input type="hidden" name="currentpass" value="" />
<input type="hidden" name="newpass_1"   value="" />
<input type="hidden" name="newpass_2"   value="" />
</form>

<form name="client" >
<table>
<tr><td align="right"> Current password:                      </td>
    <td><input type="password" name="currentpass" size="10"/> </td></tr>
<tr><td align="right"> New password:                          </td>
    <td><input type="password" name="newpass_1" size="10"  /> </td></tr>
<tr><td align="right"> Confirm password:                      </td>
    <td><input type="password" name="newpass_2" size="10"  /> </td></tr>
<tr><td colspan="2" align="center">
    <input type="button" value="Change Password" onClick="send();">
</table>
<input type="hidden" name="ukey_cpass"  value="$ukey_cpass" />
<input type="hidden" name="lkey_cpass"  value="$lkey_cpass" />
<input type="hidden" name="ukey_npass1" value="$ukey_npass1" />
<input type="hidden" name="lkey_npass1" value="$lkey_npass1" />
<input type="hidden" name="ukey_npass2" value="$ukey_npass2" />
<input type="hidden" name="lkey_npass2" value="$lkey_npass2" />
</form>
</p>
ENDFORM
    #
    return;
}

sub verify_and_change_password {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my $homeserver = $ENV{'user.home'};
    my $currentauth=&Apache::lonnet::queryauthenticate($user,$domain);
    # Check for authentication types that allow changing of the password.
    return if ($currentauth !~ /^(unix|internal):/);
    #
    $r->print(<<ENDHEADER);
<html>
<head>
<title>LON-CAPA Preferences:  Change password for $user</title>
</head>
ENDHEADER
    #
    my $currentpass = $ENV{'form.currentpass'}; 
    my $newpass1    = $ENV{'form.newpass_1'}; 
    my $newpass2    = $ENV{'form.newpass_2'};
    my $logtoken    = $ENV{'form.logtoken'};
    # Check for empty data 
    unless (defined($currentpass) && 
	    defined($newpass1)    && 
	    defined($newpass2)    ){
	&passwordchanger($r,"<p>\n<font color='#ff0000'>ERROR</font>".
			 "Password data was blank.\n</p>");
	return;
    }
    # Get the keys
    my $lonhost = $r->dir_config('lonHostID');
    my $tmpinfo = Apache::lonnet::reply('tmpget:'.$logtoken,$lonhost);
    if (($tmpinfo=~/^error/) || ($tmpinfo eq 'con_lost')) {
        # I do not a have a better idea about how to handle this
	$r->print(<<ENDERROR);
<p>
<font color="#ff0000">ERROR:</font> Unable to retrieve stored token for
password decryption.  Please log out and try again.
</p>
ENDERROR
        # Probably should log an error here
        return;
    }
    my ($ckey,$n1key,$n2key)=split(/&/,$tmpinfo);
    # 
    $currentpass = &des_decrypt($ckey ,$currentpass);
    $newpass1    = &des_decrypt($n1key,$newpass1);
    $newpass2    = &des_decrypt($n2key,$newpass2);
    # 
    if ($newpass1 ne $newpass2) {
	&passwordchanger($r,
			 '<font color="#ff0000">ERROR:</font>'.
			 'The new passwords you entered do not match.  '.
			 'Please try again.');
	return;
    }
    if (length($newpass1) < 7) {
	&passwordchanger($r,
			 '<font color="#ff0000">ERROR:</font>'.
			 'Passwords must be a minimum of 7 characters long.  '.
			 'Please try again.');
	return;
    }
    #
    # Check for bad characters
    my $badpassword = 0;
    foreach (split(//,$newpass1)) {
	$badpassword = 1 if ((ord($_)<32)||(ord($_)>126));
    }
    if ($badpassword) {
	# I can't figure out how to enter bad characters on my browser.
	&passwordchanger($r,<<ENDERROR);
<font color="#ff0000">ERROR:</font>
The password you entered contained illegal characters.<br />
Valid characters are: space and <br />
<pre>
!&quot;\#$%&amp;\'()*+,-./0123456789:;&lt;=&gt;?\@
ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_\`abcdefghijklmnopqrstuvwxyz{|}~
</pre>
ENDERROR
    }
    # 
    # Change the password (finally)
    my $result = &Apache::lonnet::changepass
	($user,$domain,$currentpass,$newpass1,$homeserver);
    # Inform the user the password has (not?) been changed
    if ($result =~ /^ok$/) {
	$r->print(<<"ENDTEXT");
<h2>The password for $user was successfully changed</h2>
ENDTEXT
    } else {
	# error error: run in circles, scream and shout
        $r->print(<<ENDERROR);
<h2><font color="#ff0000">The password for $user was not changed</font></h2>
Please make sure your old password was entered correctly.
ENDERROR
    }
    return;
}

################################################################
#            discussion display subroutines 
################################################################
sub discussionchanger {
    my $r = shift;
    my $user       = $ENV{'user.name'};
    my $domain     = $ENV{'user.domain'};
    my %userenv = &Apache::lonnet::get
        ('environment',['discdisplay','discmarkread']);
    my $discdisp = 'allposts';
    my $discmark = 'onmark';

    if (defined($userenv{'discdisplay'})) {
        unless ($userenv{'discdisplay'} eq '') { 
            $discdisp = $userenv{'discdisplay'};
        }
    }
    if (defined($userenv{'discmarkread'})) {
        unless ($userenv{'discdisplay'} eq '') { 
            $discmark = $userenv{'discmarkread'};
        }
    }

    my $newdisp = 'unread';
    my $newmark = 'ondisp';

    my $function = &Apache::loncommon::get_users_function();
    my $color = &Apache::loncommon::designparm($function.'.tabbg',
                                                    $ENV{'user.domain'});
    my %lt = &Apache::lonlocal::texthash(
        'pref' => 'Display Preference',
        'curr' => 'Current setting ',
        'actn' => 'Action',
        'sdpf' => 'Set display preferences for discussion posts for both bulletin boards and individual resources in all your courses.',
        'prca' => 'Preferences can be set that determine',
        'whpo' => 'Which posts are displayed when you display a bulletin board or resource, and',
        'unwh' => 'Under what circumstances posts are identfied as "New"',
        'allposts' => 'All posts',
        'unread' => 'New posts only',
        'ondisp' => 'Once displayed',
        'onmark' => 'Once marked as read',
        'disa' => 'Posts displayed?',
        'npmr' => 'New posts cease to be identified as "New"?',
        'thde'  => 'The preferences you set here can be overridden within each individual discussion.',
        'chgt' => 'Change to '
    );
    my $dispchange = $lt{'unread'};
    my $markchange = $lt{'ondisp'};
    my $currdisp = $lt{'allposts'};
    my $currmark = $lt{'onmark'};

    if ($discdisp eq 'unread') {
        $dispchange = $lt{'allposts'};
        $currdisp = $lt{'unread'};
        $newdisp = 'allposts';
    }

    if ($discmark eq 'ondisp') {
        $markchange = $lt{'onmark'};
        $currmark = $lt{'ondisp'};
        $newmark = 'onmark';
    }
    
    $r->print(<<"END");
<form name="server" action="/adm/preferences" method="post">
<input type="hidden" name="action" value="verify_and_change_discussion" />
<br />
$lt{'sdpf'}<br/> $lt{'prca'}  <ol><li>$lt{'whpo'}</li><li>$lt{'unwh'}</li></ol> 
<br />
<br />
<table border="0" cellpadding="0" cellspacing="0">
 <tr>
  <td width="100%" bgcolor="#000000">
   <table width="100%" border="0" cellpadding="1" cellspacing="0">
    <tr>
     <td width="100%" bgcolor="#000000">
      <table border="0" cellpadding="3" cellspacing="3" bgcolor="#FFFFFF">
       <tr bgcolor="$color">
        <td><b>$lt{'pref'}</b></td>
        <td><b>$lt{'curr'}</b></td>
        <td><b>$lt{'actn'}?</b></td>
       </tr>
       <tr bgcolor="#dddddd">
       <td>$lt{'disa'}</td>
       <td>$lt{$discdisp}</td>
       <td><input type="checkbox" name="discdisp" /><input type="hidden" name="newdisp" value="$newdisp" />&nbsp;$lt{'chgt'} "$dispchange"</td>
      </tr><tr bgcolor="#eeeeee">
       <td>$lt{'npmr'}</td>
       <td>$lt{$discmark}</td>
       <td><input type="checkbox" name="discmark" /><input type="hidden" name="newmark" value="$newmark" />&nbsp;$lt{'chgt'} "$markchange"</td>
      </tr>
     </table>
    </td>
   </tr>
  </table>
 </td>
</tr>
</table>
<br />
<br />
<input type="submit" name="sub" value="Store Changes" />
<br />
<br />
Note: $lt{'thde'}
</form>
END
}
                                                                                                                
sub verify_and_change_discussion {
    my $r = shift;
    my $user     = $ENV{'user.name'};
    my $domain   = $ENV{'user.domain'};
    my $message='';
    if (defined($ENV{'form.discdisp'}) ) {
        my $newdisp  = $ENV{'form.newdisp'};
        if ($newdisp eq 'unread') {
            $message .='In discussions: only new posts will be displayed.<br/>';
            &Apache::lonnet::put('environment',{'discdisplay' => $newdisp});
            &Apache::lonnet::appenv('environment.discdisplay' => $newdisp);
        } else {
            $message .= 'In discussions: all posts will be displayed.<br/>';
            &Apache::lonnet::del('environment',['discdisplay']);
            &Apache::lonnet::delenv('environment\.discdisplay');
        }
    }
    if (defined($ENV{'form.discmark'}) ) {
        my $newmark = $ENV{'form.newmark'};
        if ($newmark eq 'ondisp') {
           $message.='In discussions: new posts will be cease to be identified as "new" after display.<br/>';
            &Apache::lonnet::put('environment',{'discmarkread' => $newmark});
            &Apache::lonnet::appenv('environment.discmarkread' => $newmark);
        } else {
            $message.='In discussions: posts will be identified as "new" until marked as read by the reader.<br/>';
            &Apache::lonnet::del('environment',['discmarkread']);
            &Apache::lonnet::delenv('environment\.discmarkread');
        }
    }
    $r->print(<<ENDVCSCREEN);
</p>
$message
ENDVCSCREEN
}

######################################################
#            other handler subroutines               #
######################################################

################################################################
#                          Main handler                        #
################################################################
sub handler {
    my $r = shift;
    my $user = $ENV{'user.name'};
    my $domain = $ENV{'user.domain'};
    &Apache::loncommon::content_type($r,'text/html');
    # Some pages contain DES keys and should not be cached.
    &Apache::loncommon::no_cache($r);
    $r->send_http_header;
    return OK if $r->header_only;
    #
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['action','wysiwyg','returnurl']);
    #
    &Apache::lonhtmlcommon::clear_breadcrumbs();
    &Apache::lonhtmlcommon::add_breadcrumb
        ({href => '/adm/preferences',
          text => 'Set User Preferences'});

    my @Options;
    # Determine current authentication method
    my $currentauth=&Apache::lonnet::queryauthenticate($user,$domain);
    if ($currentauth =~ /^(unix|internal):/) {
        push (@Options,({ action   => 'changepass',
                          linktext => 'Change Password',
                          href     => '/adm/preferences',
                          help     => 'Change_Password',
                          subroutine => \&passwordchanger,
                          breadcrumb => 
                              { href => '/adm/preferences?action=changepass',
                                text => 'Change Password'},
                          },
                        { action => 'verify_and_change_pass',
                          subroutine => \&verify_and_change_password,
                          breadcrumb => 
                              { href =>'/adm/preferences?action=changepass',
                                text => 'Change Password'},
                          printmenu => 'yes',
                          }));
    }
    push (@Options,({ action   => 'changescreenname',
                      linktext => 'Change Screen Name',
                      href     => '/adm/preferences',
                      help     => 'Prefs_Screen_Name_Nickname',
                      subroutine => \&screennamechanger,
                      breadcrumb => 
                          { href => '/adm/preferences?action=changescreenname',
                            text => 'Change Screen Name'},
                      },
                    { action   => 'verify_and_change_screenname',
                      subroutine => \&verify_and_change_screenname,
                      breadcrumb => 
                          { href => '/adm/preferences?action=changescreenname',
                            text => 'Change Screen Name'},
                      printmenu => 'yes',
                      }));

    push (@Options,({ action   => 'changemsgforward',
                      linktext => 'Change Message Forwarding',
                      text     => 'and Notification Addresses',
                      href     => '/adm/preferences',
                      help     => 'Prefs_Forwarding',
                      breadcrumb => 
                          { href => '/adm/preferences?action=changemsgforward',
                            text => 'Change Message Forwarding'},
                      subroutine => \&msgforwardchanger,
                      },
                    { action => 'verify_and_change_msgforward',
                      breadcrumb => 
                          { href => '/adm/preferences?action=changemsgforward',
                            text => 'Change Message Forwarding'},
                      printmenu => 'yes',
                      subroutine => \&verify_and_change_msgforward }));
    my $aboutmeaction=
        '/adm/'.$ENV{'user.domain'}.'/'.$ENV{'user.name'}.'/aboutme';
    push (@Options,{ action => 'none', 
                     linktext =>
                         q{Edit the 'About Me' Personal Information Screen},
                     href => $aboutmeaction});
    push (@Options,({ action => 'changecolors',
                      linktext => 'Change Color Scheme',
                      href => '/adm/preferences',
                      help => 'Change_Colors',
                      breadcrumb => 
                          { href => '/adm/preferences?action=changecolors',
                            text => 'Change Colors'},
                      subroutine => \&colorschanger,
                  },
                    { action => 'verify_and_change_colors',
                      breadcrumb => 
                          { href => '/adm/preferences?action=changecolors',
                            text => 'Change Colors'},
                      printmenu => 'yes',
                      subroutine => \&verify_and_change_colors,
                      }));
    push (@Options,({ action => 'changelanguages',
                      linktext => 'Change Language Preferences',
                      href => '/adm/preferences',
                      breadcrumb=>
                          { href => '/adm/preferences?action=changelanguages',
                            text => 'Change Language'},
                      subroutine =>  \&languagechanger,
                  },
                    { action => 'verify_and_change_languages',
                      breadcrumb=>
                          {href => '/adm/preferences?action=changelanguages',
                           text => 'Change Language'},
                      printmenu => 'yes',
                      subroutine=>\&verify_and_change_languages, }
                    ));
    push (@Options,({ action => 'changewysiwyg',
                      linktext => 'Change WYSIWYG Editor Preferences',
                      href => '/adm/preferences',
                      breadcrumb => 
                            { href => '/adm/preferences?action=changewysiwyg',
                              text => 'Change WYSIWYG Preferences'},
                      subroutine => \&wysiwygchanger,
                  },
                    { action => 'set_wysiwyg',
                      breadcrumb =>
                          { href => '/adm/preferences?action=changewysiwyg',
                            text => 'Change WYSIWYG Preferences'},
                      printmenu => 'yes',
                      subroutine => \&verify_and_change_wysiwyg, }
                    ));
    push (@Options,({ action => 'changediscussions',
                      linktext => 'Change Discussion Display Preferences',
                      href => '/adm/preferences',
                      breadcrumb => 
                            { href => '/adm/preferences?action=changediscussions',
                              text => 'Change Discussion Preferences'},
                      subroutine => \&discussionchanger,
                  },
                    { action => 'verify_and_change_discussion',
                      breadcrumb =>
                          { href => '/adm/preferences?action=changediscussions',
                            text => 'Change Discussion Preferences'},
                      printmenu => 'yes',
                      subroutine => \&verify_and_change_discussion, }
                    ));
                       
    if ($ENV{'user.name'} =~ /^(albertel|koretemey|korte|hallmat3|turtle)$/) {
        push (@Options,({ action => 'debugtoggle',
                          printmenu => 'yes',
                          subroutine => \&toggle_debug,
                          }));
    }
    $r->print(<<ENDHEADER);
<html>
<head>
<title>LON-CAPA Preferences</title>
</head>
ENDHEADER
    my $call = undef;
    my $printmenu = 'yes';
    foreach my $option (@Options) {
        if ($option->{'action'} eq $ENV{'form.action'}) {
            $call = $option->{'subroutine'};
            $printmenu = $option->{'printmenu'};
            if (exists($option->{'breadcrumb'})) {
                &Apache::lonhtmlcommon::add_breadcrumb
                    ($option->{'breadcrumb'});
            }
        }
    }
    $r->print(&Apache::loncommon::bodytag('Change Preferences'));
    $r->print(&Apache::lonhtmlcommon::breadcrumbs
              (undef,'Change Preferences'));
    if (defined($call)) {
        $call->($r);
    }
    if (($printmenu eq 'yes') && (!$ENV{'form.returnurl'})) {
        my $optionlist = '<table cellpadding="5">';
        if ($ENV{'user.name'} =~ 
                         /^(albertel|kortemey|korte|hallmat3|turtle)$/
            ) {
            push (@Options,({ action => 'debugtoggle',
                              linktext => 'Toggle Debug Messages',
                              text => 'Current Debug status is -'.
                                  $ENV{'user.debug'}.'-.',
                              href => '/adm/preferences',
                              printmenu => 'yes',
                              subroutine => \&toggle_debug,
                              }));
        }
        foreach my $option(@Options) {
            my $optiontext = '';
            if (exists($option->{'href'})) {
                $optiontext .= 
                    '<a href="'.$option->{'href'}.
                    '?action='.$option->{'action'}.'">'.
                    $option->{'linktext'}.'</a>';
            }
            if (exists($option->{'text'})) {
                $optiontext .= ' '.$option->{'text'};
            }
            if ($optiontext ne '') {
                $optiontext = '<font size="+1">'.$optiontext.'</font>'; 
                my $helplink = '&nbsp;';
                if (exists($option->{'help'})) {
                    $helplink = &Apache::loncommon::help_open_topic
                                                    ($option->{'help'});
                }
                $optionlist .= '<tr>'.
                    '<td>'.$helplink.'</td>'.
                    '<td>'.$optiontext.'</td>'.
                    '</tr>';
            }
        }
        $optionlist .= '</table>';
        $r->print($optionlist);
    } elsif ($ENV{'form.returnurl'}) {
	$r->print('<br /><a href="'.$ENV{'form.returnurl'}.'"><font size="+1">'.
		  &mt('Return').'</font></a>');
    }
    $r->print(<<ENDFOOTER);
</body>
</html>
ENDFOOTER
    return OK;
}

sub toggle_debug {
    if ($ENV{'user.debug'}) {
        &Apache::lonnet::delenv('user\.debug');
    } else {
        &Apache::lonnet::appenv('user.debug' => 1);
    }
}

1;
__END__
